@@ -2,8 +2,8 @@
  *            main.c
  *
  *  Thu Jul 15 06:01:04 2004
- *  Last Update: 03/12/2005
- *  Copyright  2004 - 2005  bountykiller
+ *  Last Update: 01/04/2007
+ *  Copyright  2004 - 2007  bountykiller
  *  Email: masse_nicolas@yahoo.fr
  ****************************************************************************/
 
@@ -28,9 +28,13 @@
 
 #include "types.h"
 #include "callbacks.h"
+#include "avoid_deprecation.h"
 #include <glib/gprintf.h>
 #include <string.h>
 
+//TO DO:
+//replace xfce_icon_theme_load_category and friends
+
 
 /* Quicklauncher funcs */
 
@@ -39,65 +43,67 @@ gboolean quicklauncher_load_config(t_quicklauncher *quicklauncher, const gchar*
 void quicklauncher_save_config(t_quicklauncher *quicklauncher, const gchar* filename);
 
 /* Launcher funcs */
+gboolean launcher_passthrough (GtkWidget *box, GdkEventCrossing *event, t_launcher *launcher);
+gboolean launcher_clicked (GtkWidget *box, GdkEventButton *event, t_launcher *launcher);
 t_launcher* launcher_load_config(XfceRc *rcfile, gint num, t_quicklauncher *quicklauncher);
 void launcher_save_config(t_launcher *launcher, XfceRc *rcfile, guint16 num);
 
 /* -------------------------------------------------------------------- *
- *                     Panel Plugin Interface               *
+ *	                     	Panel Plugin Interface               		*
  * -------------------------------------------------------------------- */
 
-static void 
+static void
 quicklauncher_construct (XfcePanelPlugin *plugin);
 
-static void 
+static void
 quicklauncher_orientation_changed(XfcePanelPlugin *plugin,
 									GtkOrientation orientation,
                                     t_quicklauncher *quicklauncher);
-static gboolean 
+static gboolean
 quicklauncher_set_size(XfcePanelPlugin *plugin,gint size,
 						t_quicklauncher *quicklauncher);
 
-static void 
+static void
 quicklauncher_free_data(XfcePanelPlugin *plugin,t_quicklauncher *quicklauncher);
 
-static void 
+static void
 quicklauncher_save(XfcePanelPlugin *plugin, t_quicklauncher *quicklauncher);
 
-static void 
+static void
 quicklauncher_configure(XfcePanelPlugin *plugin, t_quicklauncher *quicklauncher);
 
-static void 
+static void
 quicklauncher_about(XfcePanelPlugin *plugin, t_quicklauncher *quicklauncher);
 
 XFCE_PANEL_PLUGIN_REGISTER_INTERNAL(quicklauncher_construct);
 
 /* create widgets and connect to signals */
-static void 
+static void
 quicklauncher_construct (XfcePanelPlugin *plugin)
 {
     t_quicklauncher *quicklauncher = quicklauncher_new(plugin);
     xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
-    g_signal_connect (plugin, "orientation-changed", 
+    g_signal_connect (plugin, "orientation-changed",
                       G_CALLBACK (quicklauncher_orientation_changed), quicklauncher);
-    
-    g_signal_connect (plugin, "size-changed", 
+
+    g_signal_connect (plugin, "size-changed",
                       G_CALLBACK (quicklauncher_set_size), quicklauncher);
-    
-    g_signal_connect (plugin, "free-data", 
+
+    g_signal_connect (plugin, "free-data",
                       G_CALLBACK (quicklauncher_free_data), quicklauncher);
-    
-    g_signal_connect (plugin, "save", 
+
+    g_signal_connect (plugin, "save",
                       G_CALLBACK (quicklauncher_save), quicklauncher);
-    
-    xfce_panel_plugin_menu_show_configure (plugin);
-    g_signal_connect (plugin, "configure-plugin", 
+
+    xfce_panel_plugin_menu_show_configure(plugin);
+    g_signal_connect (plugin, "configure-plugin",
                       G_CALLBACK (quicklauncher_configure), quicklauncher);
 
    xfce_panel_plugin_menu_show_about(plugin);
-   g_signal_connect (plugin, "about", 
+   g_signal_connect (plugin, "about",
                       G_CALLBACK (quicklauncher_about), quicklauncher);
-   
+
 }
 
 
@@ -114,16 +120,9 @@ gboolean quicklauncher_set_size(XfcePanelPlugin *plugin, gint size,
 							t_quicklauncher *quicklauncher)
 {
 	DBG ("setting size %d", size);
-	GList *liste;
-	quicklauncher->icon_size = (int) (0.75 * size/quicklauncher->nb_lines);
-	for(liste = quicklauncher->launchers;
-		liste ; liste = g_list_next(liste) )
-	{
-		launcher_update_icon((t_launcher*)liste->data, quicklauncher->icon_size);		
-		gtk_container_set_border_width( GTK_CONTAINER( ( (t_launcher*)liste->data)->widget), 
-										(int)quicklauncher->icon_size/8);
-		//printf("%d %d \n",(int) 0.75 * size/quicklauncher->nb_lines, (int)quicklauncher->icon_size/8);
-	}
+	quicklauncher->icon_size = (int) (0.70 * size/quicklauncher->nb_lines);
+	g_list_foreach(quicklauncher->launchers, (GFunc)launcher_update_icon, 
+					(gpointer)quicklauncher->icon_size);
 	return TRUE;
 }
 
@@ -143,18 +142,19 @@ void quicklauncher_save(XfcePanelPlugin *plugin, t_quicklauncher *quicklauncher)
 	}
 }
 
-void 
+void
 quicklauncher_configure(XfcePanelPlugin *plugin, t_quicklauncher *quicklauncher)
 {
-	xfce_panel_plugin_block_menu(plugin); 
-	t_qck_launcher_opt_dlg* dlg = create_qck_launcher_dlg();
+	t_qck_launcher_opt_dlg* dlg;
+	xfce_panel_plugin_block_menu(plugin);
+	dlg = create_qck_launcher_dlg();
 	qck_launcher_opt_dlg_set_quicklauncher(quicklauncher);
 	gtk_dialog_run(GTK_DIALOG(dlg->dialog));
 	xfce_panel_plugin_unblock_menu(plugin);
 }
 
 
-void 
+void
 quicklauncher_about(XfcePanelPlugin *plugin, t_quicklauncher *quicklauncher)
 {
 	GtkWidget *about;
@@ -167,20 +167,19 @@ quicklauncher_about(XfcePanelPlugin *plugin, t_quicklauncher *quicklauncher)
 	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about), "http://xfce-goodies.berlios.de");
 	gtk_about_dialog_set_website_label(GTK_ABOUT_DIALOG(about), _("Other plugins available here"));
 	gtk_dialog_run(GTK_DIALOG(about));
-	gtk_widget_destroy (about); 
+	gtk_widget_destroy (about);
 }
 
 
 /* -------------------------------------------------------------------- *
- *              quicklauncher utility funcs               *
+ *          		  	  quicklauncher utility funcs      		        *
  * -------------------------------------------------------------------- */
 
 void
 quicklauncher_add_element(t_quicklauncher *quicklauncher, t_launcher *launcher)
 {
 	quicklauncher->launchers = g_list_append(quicklauncher->launchers, (gpointer)launcher);
-	xfce_panel_plugin_add_action_widget(quicklauncher->plugin, launcher->widget);
-	quicklauncher->nb_launcher++; 
+	quicklauncher->nb_launcher++;
 }
 
 
@@ -199,18 +198,18 @@ quicklauncher_remove_element(t_quicklauncher *quicklauncher, gint num)
 void
 quicklauncher_organize(t_quicklauncher *quicklauncher)
 {
-	DBG ("Organize quicklauncher");
-	gint i, j, launch_per_line, nb_lines;
+	gint i, j, launch_per_line, nb_lines, extra_space;
 	GList *toplace;
-	
-	g_assert( (!quicklauncher->table || GTK_IS_TABLE(quicklauncher->table)) && GTK_IS_CONTAINER(quicklauncher->plugin));
-	if (quicklauncher->launchers) 
+
+	DBG ("Organize quicklauncher");
+	g_return_if_fail( (!quicklauncher->table || GTK_IS_TABLE(quicklauncher->table)) && GTK_IS_CONTAINER(quicklauncher->plugin));
+	if (quicklauncher->launchers)
 	{
-		nb_lines = MIN(quicklauncher->nb_lines, quicklauncher->nb_launcher); 
+		nb_lines = MIN(quicklauncher->nb_lines, quicklauncher->nb_launcher);
 		toplace = g_list_first(quicklauncher->launchers);
 		if(quicklauncher->nb_launcher % quicklauncher->nb_lines == 0)
 			launch_per_line = quicklauncher->nb_launcher / quicklauncher->nb_lines;
-		else 
+		else
 			launch_per_line = quicklauncher->nb_launcher / quicklauncher->nb_lines+1;
 		if (quicklauncher->orientation != GTK_ORIENTATION_HORIZONTAL)
 		{
@@ -218,19 +217,23 @@ quicklauncher_organize(t_quicklauncher *quicklauncher)
 			nb_lines = launch_per_line;
 			launch_per_line = i;
 		}
-		if (quicklauncher->table)
-			gtk_table_resize(GTK_TABLE(quicklauncher->table), nb_lines, launch_per_line);
-		
+		gtk_table_resize(GTK_TABLE(quicklauncher->table), nb_lines, launch_per_line);
 		j = quicklauncher->nb_launcher;
 		for (i=0; i < nb_lines; ++i)
 		{
 			for(j=0; (j < launch_per_line) && (toplace); ++j, toplace = g_list_next(toplace))
 			{
-				g_assert(toplace && GTK_IS_WIDGET(((t_launcher*)toplace->data)->widget) );
-				gtk_table_attach_defaults( GTK_TABLE(quicklauncher->table),
-											((t_launcher*)toplace->data)->widget,
-											j, j+1, i, i+1);
-				//gtk_container_add (GTK_CONTAINER (quicklauncher->hbox[i]), ((t_launcher*)toplace->data)->widget);
+				g_return_if_fail(toplace);
+				DBG ("To place: %s %d", ((t_launcher*)toplace->data)->command, (gint)((t_launcher*)toplace->data)->box);
+				if(quicklauncher->extra_spacing)
+				{
+					extra_space = (gint) (quicklauncher->icon_size * quicklauncher->extra_spacing);
+					gtk_table_attach(GTK_TABLE(quicklauncher->table), ((t_launcher*)toplace->data)->box,
+									 j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, extra_space, extra_space);
+				}
+				else
+					gtk_table_attach_defaults(GTK_TABLE(quicklauncher->table), 
+												((t_launcher*)toplace->data)->box, j, j+1, i, i+1);
 			}
 		}
 	}
@@ -240,14 +243,18 @@ quicklauncher_organize(t_quicklauncher *quicklauncher)
 void
 quicklauncher_empty_widgets(t_quicklauncher *quicklauncher)
 {
-	GList *launcher;	
+	GList *launcher;
 	if (quicklauncher->table)
 	{
-		for( launcher = g_list_first(quicklauncher->launchers);
-			 launcher; launcher = g_list_next(launcher) )
-			gtk_container_remove(GTK_CONTAINER(quicklauncher->table), 
-								((t_launcher*)launcher->data)->widget);
-	}	
+		for(launcher = g_list_first(quicklauncher->launchers);
+			launcher; launcher = g_list_next(launcher) )
+		{
+			DBG ("remove %s %d", ((t_launcher*)launcher->data)->command, (gint)(((t_launcher*)launcher->data)->box));
+			gtk_container_remove(GTK_CONTAINER(quicklauncher->table), ((t_launcher*)launcher->data)->box);
+			DBG ("Is the box alive? %s", 
+				GTK_IS_WIDGET(((t_launcher*)launcher->data)->box)?"YES":"NO");
+		}
+	}
 }
 
 
@@ -266,7 +273,7 @@ quicklauncher_empty(t_quicklauncher *quicklauncher)
 }
 
 
-void 
+void
 quicklauncher_set_nblines(t_quicklauncher *quicklauncher, gint nb_lines)
 {
 	if (nb_lines != quicklauncher->nb_lines)
@@ -274,55 +281,80 @@ quicklauncher_set_nblines(t_quicklauncher *quicklauncher, gint nb_lines)
 		quicklauncher_empty_widgets(quicklauncher);
 		quicklauncher->nb_lines = nb_lines;
 		quicklauncher_set_size(quicklauncher->plugin,
-			xfce_panel_plugin_get_size(quicklauncher->plugin),quicklauncher);
+					xfce_panel_plugin_get_size(quicklauncher->plugin),quicklauncher);
 		quicklauncher_organize(quicklauncher);
 	}
 }
 
+void quicklauncher_set_extra_spacing(t_quicklauncher *quicklauncher, gdouble spacing)
+{
+	quicklauncher->extra_spacing = MIN(spacing, 1.0);
+	quicklauncher_empty_widgets(quicklauncher);
+	quicklauncher_organize(quicklauncher);
+}
+
+void quicklauncher_set_has_tooltips(t_quicklauncher *quicklauncher, gboolean value)
+{
+	if(quicklauncher->has_tooltips == value) return;
+	quicklauncher->has_tooltips = value;
+	g_list_foreach(quicklauncher->launchers, (GFunc)launcher_set_has_tooltip, (gpointer)value);
+}
+
+
+void quicklauncher_set_has_labels(t_quicklauncher *quicklauncher, gboolean value)
+{
+	if(quicklauncher->has_labels == value) return;
+	quicklauncher->has_labels = value;
+	g_list_foreach(quicklauncher->launchers, (GFunc)launcher_set_has_label, (gpointer)value);
+}
+
 
 void
 quicklauncher_load_default(t_quicklauncher *quicklauncher)
 {
 	t_launcher *launcher;
 	quicklauncher->nb_lines = 2;
-	
-	launcher = launcher_new("xflock4", XFCE_ICON_CATEGORY_SYSTEM, 
+	quicklauncher->has_labels = FALSE;
+	quicklauncher->has_tooltips = TRUE;
+
+	launcher = launcher_new(_("Lock"), "xflock4", XFCE_ICON_CATEGORY_SYSTEM,
 							NULL, quicklauncher);
 	quicklauncher_add_element(quicklauncher, launcher);
-	launcher = launcher_new("xfce-setting-show", XFCE_ICON_CATEGORY_SETTINGS, 
+	launcher = launcher_new(_("Parameters"),"xfce-setting-show", XFCE_ICON_CATEGORY_SETTINGS,
 							NULL, quicklauncher);
 	quicklauncher_add_element(quicklauncher, launcher);
-	launcher = launcher_new("xfce4-appfinder", XFCE_ICON_CATEGORY_UTILITY, 
+	launcher = launcher_new(_("Applications"), "xfce4-appfinder", XFCE_ICON_CATEGORY_UTILITY,
 							NULL, quicklauncher);
 	quicklauncher_add_element(quicklauncher, launcher);
-	launcher = launcher_new("xfhelp4", XFCE_ICON_CATEGORY_HELP, 
+	launcher = launcher_new(_("Help"), "xfhelp4", XFCE_ICON_CATEGORY_HELP,
 							NULL, quicklauncher);
 	quicklauncher_add_element(quicklauncher, launcher);
-	//g_assert(quicklauncher->nb_launcher == 4);
-}	
+	//g_return_if_fail(quicklauncher->nb_launcher == 4);
+}
 
 
 t_quicklauncher *
 quicklauncher_new (XfcePanelPlugin *plugin)
 {
-	DBG ("create quicklauncher");
 	t_quicklauncher *quicklauncher;
+	gchar *filename;
 	
+	DBG ("create quicklauncher");
 	quicklauncher = g_new0(t_quicklauncher, 1);
-	gchar *filename = xfce_panel_plugin_save_location(plugin, TRUE);
-	
-	if((!filename) || (!quicklauncher_load_config(quicklauncher, filename) ) )
-		quicklauncher_load_default(quicklauncher);
-	
-	quicklauncher->icon_size = (gint) 0.75 * xfce_panel_plugin_get_size(plugin)/2;
+	filename = xfce_panel_plugin_save_location(plugin, TRUE);
+	quicklauncher->icon_size = (gint) (0.70 * xfce_panel_plugin_get_size(plugin)/2);
+	DBG ("icon size: %d", quicklauncher->icon_size);
 	quicklauncher->orientation = xfce_panel_plugin_get_orientation(plugin);
 	quicklauncher->plugin = plugin;
-	quicklauncher->table = g_object_ref(gtk_table_new(2, 2, TRUE));
+	quicklauncher->_last_zoomed_launcher = NULL;
+	gtk_widget_add_events(GTK_WIDGET(plugin),GDK_POINTER_MOTION_MASK);
+	quicklauncher->table = g_object_ref(gtk_table_new(2, 2, FALSE));
 	gtk_table_set_col_spacings(GTK_TABLE(quicklauncher->table), 0);
-	gtk_container_add( GTK_CONTAINER(quicklauncher->plugin), quicklauncher->table);
-	xfce_panel_plugin_add_action_widget(quicklauncher->plugin, quicklauncher->table);
+	gtk_table_set_row_spacings(GTK_TABLE(quicklauncher->table), 0);
+	gtk_container_add(GTK_CONTAINER(quicklauncher->plugin), quicklauncher->table);
 	gtk_widget_show(quicklauncher->table);
-	
+	if((!filename) || (!quicklauncher_load_config(quicklauncher, filename) ) )
+		quicklauncher_load_default(quicklauncher);
 	quicklauncher_organize(quicklauncher);
 	return quicklauncher;
 }
@@ -333,7 +365,6 @@ quicklauncher_free(t_quicklauncher *quicklauncher)
 {
 	g_list_foreach(quicklauncher->launchers, (GFunc) launcher_free, NULL);
 	g_list_free(quicklauncher->launchers);
-	
 	g_object_unref(quicklauncher->table);
 	g_free(quicklauncher);
 }
@@ -341,118 +372,127 @@ quicklauncher_free(t_quicklauncher *quicklauncher)
 
 gboolean quicklauncher_load_config(t_quicklauncher *quicklauncher, const gchar* filename)
 {
-	
+
 	XfceRc* rcfile;
+	gint i;
+	
 	if( (rcfile = xfce_rc_simple_open(filename, TRUE) ))
 	{
 		xfce_rc_set_group(rcfile, NULL);
 		quicklauncher->nb_lines = xfce_rc_read_int_entry(rcfile, "nb_lines", 1);
-		gint i = xfce_rc_read_int_entry(rcfile, "nb_launcher", 0);
-		g_assert(i >= 0);
+		quicklauncher->extra_spacing = ((gdouble)xfce_rc_read_int_entry(rcfile, "extra_spacing", 0))/100;
+		quicklauncher->has_tooltips = xfce_rc_read_bool_entry(rcfile, "has_tooltips", TRUE);
+		quicklauncher->has_labels = xfce_rc_read_bool_entry(rcfile, "has_labels", FALSE);
+		i = xfce_rc_read_int_entry(rcfile, "nb_launcher", 0);
+		g_return_val_if_fail(i >= 0, FALSE);
 		while(i)
 		{
 			t_launcher *launcher = launcher_load_config(rcfile, i, quicklauncher);
 			quicklauncher_add_element(quicklauncher, launcher);
 			i--;
-			if(!i)	
+			if(!i)
 				return TRUE;
 		}
-	} 
+	}
 	return FALSE;
 }
 
 void
 quicklauncher_save_config(t_quicklauncher *quicklauncher, const gchar* filename)
 {
-	guint16 i = quicklauncher->nb_launcher; //hope it always works
-	XfceRc* rcfile = xfce_rc_simple_open(filename, FALSE);
+	XfceRc* rcfile;
+	GList* liste;
+	guint16 i;
+	//guint16 i = quicklauncher->nb_launcher; //hope it always works ==> seems that it does not ;)
+	DBG("saving config in %s", filename);
+	i = quicklauncher->nb_launcher;
+	rcfile = xfce_rc_simple_open(filename, FALSE);
 	if(!rcfile) return;
 	
 	xfce_rc_set_group(rcfile, NULL);
 	xfce_rc_write_int_entry(rcfile, "nb_lines", quicklauncher->nb_lines);
 	xfce_rc_write_int_entry(rcfile, "nb_launcher", quicklauncher->nb_launcher);
+	xfce_rc_write_int_entry(rcfile, "extra_spacing", (gint)(quicklauncher->extra_spacing * 100));
+	xfce_rc_write_bool_entry(rcfile, "has_tooltips", quicklauncher->has_tooltips);
+	xfce_rc_write_bool_entry(rcfile, "has_labels", quicklauncher->has_labels);
 	xfce_rc_flush(rcfile);
-	GList* liste;
 	for( liste = quicklauncher->launchers; liste; liste = g_list_next(liste), --i)
 		launcher_save_config((t_launcher*)liste->data, rcfile, i);
-	
-	g_assert(i == 0);
 	xfce_rc_close(rcfile);
+	g_return_if_fail(i == 0);
 }
 
 /* -------------------------------------------------------------------- *
- *                        Launcher Interface                 *
+ *                        		Launcher Interface                		*
  * -------------------------------------------------------------------- */
 
-//TO DO: support XFCE icon by name
+//TO DO: support icon by name
 GdkPixbuf *
 _create_pixbuf(gint id, const gchar* name, gint size)
 {
-	DBG ("creating pixbuf %d", size);
-	GdkPixbuf  *pixbuf;
+	DBG ("id: %d" , id);
+	GdkPixbuf  *pixbuf = NULL;
 	if(id != XFCE_ICON_CATEGORY_EXTERN)
-		pixbuf = xfce_icon_theme_load_category(DEFAULT_ICON_THEME, id, size);
+		pixbuf = xfce_themed_icon_load_category(id, size);
 	else
 		pixbuf = gdk_pixbuf_new_from_file_at_size(name, size, size, NULL);
 	if(!pixbuf)
-		pixbuf = xfce_icon_theme_load_category(DEFAULT_ICON_THEME, XFCE_ICON_CATEGORY_UNKNOWN, size);
+		pixbuf = xfce_themed_icon_load_category(XFCE_ICON_CATEGORY_UNKNOWN, size);
 	return pixbuf;
 }
 
-gboolean
-launcher_clicked (GtkWidget *event_box, GdkEventButton *event, t_launcher *launcher)
+
+gboolean launcher_clicked (GtkWidget *box, GdkEventButton *event, t_launcher *launcher)
 {
-	int size = 1.25 * launcher->quicklauncher->icon_size;
-	if (event->button != 1) 
-		return FALSE;
-	if (event->type == GDK_BUTTON_PRESS) 
-	{ 
-		g_assert(launcher->zoomed_img);
-		if(event->x < 0 || event->x > size || event->y < 0 || event->y > size)
-			return FALSE;
+	DBG ("Launcher clicked => %s", launcher->command);
+	if (event->type == GDK_BUTTON_PRESS)
+    {        
+		g_return_val_if_fail(launcher->zoomed_img, FALSE); 
 		if (!launcher->clicked_img)
 		{
 			launcher->clicked_img = gdk_pixbuf_copy (launcher->zoomed_img);
 			gdk_pixbuf_saturate_and_pixelate(launcher->zoomed_img, launcher->clicked_img, 5, TRUE);
 		}
 		gtk_image_set_from_pixbuf (GTK_IMAGE(launcher->image), launcher->clicked_img);
+		launcher->quicklauncher->_last_zoomed_launcher = launcher;
 	}
-	else if (event->type == GDK_BUTTON_RELEASE)
-	{
-		//g_assert(launcher->clicked_img);
-		if (event->x > 0 && event->x < size && event->y > 0 && event->y < size)
+    else if (event->type == GDK_BUTTON_RELEASE)
+    {
+		if(launcher->quicklauncher->_last_zoomed_launcher == launcher)
+		{
+			g_return_val_if_fail(launcher->clicked_img, FALSE);
 			xfce_exec(launcher->command, FALSE, FALSE, NULL);
-		gtk_image_set_from_pixbuf (GTK_IMAGE(launcher->image), launcher->def_img);
-		gtk_container_set_border_width(GTK_CONTAINER (event_box),
-										(int)launcher->quicklauncher->icon_size/8);
-		//gtk_widget_set_size_request(launcher->image, size, size);
+			gtk_image_set_from_pixbuf (GTK_IMAGE(launcher->image), launcher->def_img);
+		}
+		else
+			gtk_image_set_from_pixbuf (GTK_IMAGE(launcher->quicklauncher->_last_zoomed_launcher->image),
+										launcher->def_img);
+		launcher->quicklauncher->_last_zoomed_launcher = NULL;
 	}
-	return TRUE;
+	return FALSE;
 }
 
-gboolean    
-launcher_passthrought(GtkWidget *widget, GdkEventCrossing *event, t_launcher *launcher)
+gboolean launcher_passthrough (GtkWidget *box, GdkEventCrossing *event, t_launcher *launcher)   
 {
 	if (event->type == GDK_ENTER_NOTIFY)
 	{
-		int size = 1.25 * launcher->quicklauncher->icon_size;
+		DBG ("Entred launcher %s", launcher->command);
+		int size = 1.2 * launcher->quicklauncher->icon_size;
 		if (!launcher->zoomed_img)
-			launcher->zoomed_img = gdk_pixbuf_scale_simple(launcher->def_img, size, size, GDK_INTERP_BILINEAR);
-		gtk_container_set_border_width(GTK_CONTAINER (widget), 0);
+			launcher->zoomed_img = gdk_pixbuf_scale_simple(launcher->def_img, size, size, GDK_INTERP_HYPER);
 		gtk_image_set_from_pixbuf (GTK_IMAGE(launcher->image), launcher->zoomed_img);
 	}
-	else 
+	else
 	{
+		DBG ("Exit launcher %s", launcher->command);
 		gtk_image_set_from_pixbuf (GTK_IMAGE(launcher->image), launcher->def_img);
-		gtk_container_set_border_width(GTK_CONTAINER (widget), 
-										(int)(launcher->quicklauncher->icon_size/8));
 	}
-	return TRUE;
+	return FALSE;
 }
 
 
-void 
-launcher_update_icon(t_launcher *launcher, gint size) 
+void
+launcher_update_icon(t_launcher *launcher, gint size)
 {
 	DBG ("size: %d", size);
 	UNREF(launcher->def_img);
@@ -461,73 +501,89 @@ launcher_update_icon(t_launcher *launcher, gint size)
 	launcher->def_img = _create_pixbuf(launcher->icon_id, launcher->icon_name, size);
 	if (launcher->def_img)
 		gtk_image_set_from_pixbuf(GTK_IMAGE(launcher->image), launcher->def_img);
-	gtk_widget_set_size_request(launcher->image, size, size);
+	gtk_widget_set_size_request(launcher->box, launcher->quicklauncher->has_labels?-1:size, size); 
 }
 
-void 
+void
 launcher_update_icons(gpointer data, gint* size)
 {
 	launcher_update_icon((t_launcher*)data, *size); //used with g_list_foreach as a GFunc =>could be removed
 }
 
 
-void 
-launcher_update_command(t_launcher *launcher) 
+void launcher_set_has_tooltip(t_launcher *launcher, gboolean has_tooltip)
 {
-	if (launcher->command_ids[0] )
+	DBG (has_tooltip?"Has tooltip":"No tooltip");
+	UNREF(launcher->tooltip);
+	if(has_tooltip)
 	{
-		g_signal_handler_disconnect(launcher->widget, launcher->command_ids[0]);
-		g_signal_handler_disconnect(launcher->widget, launcher->command_ids[1]);
-		g_signal_handler_disconnect(launcher->widget, launcher->command_ids[2]);
-		g_signal_handler_disconnect(launcher->widget, launcher->command_ids[3]);
+		launcher->tooltip =  g_object_ref_sink(gtk_tooltips_new());
+		gtk_tooltips_set_tip(launcher->tooltip, launcher->box, launcher->name, launcher->command);
+	}
+	else
+		launcher->tooltip = NULL;
+}
+
+
+void launcher_set_has_label(t_launcher *launcher, gboolean has_label)
+{
+	DBG (has_label?"Has label":"No label");
+	if(launcher->label)
+		gtk_widget_destroy(launcher->label);
+	if(has_label)
+	{
+		launcher->label = gtk_label_new(launcher->name);
+		gtk_box_pack_end(GTK_BOX(launcher->hbox), launcher->label, FALSE, FALSE, 0);
+		gtk_widget_show(launcher->label);
+		gtk_widget_set_size_request(launcher->box, -1, launcher->quicklauncher->icon_size); 
+	}
+	else
+	{
+		launcher->label = NULL;
+		gtk_widget_set_size_request(launcher->box, launcher->quicklauncher->icon_size, 
+									launcher->quicklauncher->icon_size);
 	}
-	gtk_tooltips_set_tip(launcher->tooltip, launcher->widget, launcher->command, launcher->command);
-	launcher->command_ids[0] = g_signal_connect(launcher->widget, "button_press_event", 
-												G_CALLBACK(launcher_clicked), launcher);
-	launcher->command_ids[1] = g_signal_connect(launcher->widget, "button-release-event", 
-												G_CALLBACK(launcher_clicked), launcher);
-	launcher->command_ids[2] = g_signal_connect(launcher->widget, "enter-notify-event", 
-												G_CALLBACK(launcher_passthrought), launcher);
-	launcher->command_ids[3] = g_signal_connect(launcher->widget, "leave-notify-event", 
-												G_CALLBACK(launcher_passthrought), launcher);
 }
 
 void create_launcher(t_launcher	*launcher)
-{	
-	launcher->widget = g_object_ref(gtk_event_box_new());
+{
+	launcher->box = g_object_ref(gtk_event_box_new());
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(launcher->box), FALSE);
+	launcher->hbox = g_object_ref(gtk_hbox_new(FALSE, 0));
+	gtk_container_add(GTK_CONTAINER(launcher->box), launcher->hbox);
+	DBG ("create launcher %s with box %d", launcher->command, (gint)launcher->box);
 	launcher->image = g_object_ref(gtk_image_new());
-	launcher->tooltip = gtk_tooltips_new();
-	gtk_container_set_border_width(GTK_CONTAINER (launcher->widget), 
-								(int)launcher->quicklauncher->icon_size/8);
-	gtk_container_add (GTK_CONTAINER (launcher->widget), launcher->image);
-	gtk_event_box_set_above_child(GTK_EVENT_BOX(launcher->widget), FALSE);
-	
+	//launcher->image = gtk_image_new();
+	gtk_box_pack_start(GTK_BOX(launcher->hbox), launcher->image, FALSE, FALSE, 0);
 	launcher_update_icon(launcher, launcher->quicklauncher->icon_size);
-	g_assert(!launcher->command_ids[0]);
-	launcher_update_command(launcher) ;
-	gtk_widget_show (launcher->image);
-	gtk_widget_show (launcher->widget);
+	gtk_widget_show(launcher->image);
+	launcher_set_has_tooltip(launcher, launcher->quicklauncher->has_tooltips);
+	launcher_set_has_label(launcher, launcher->quicklauncher->has_labels);
+	gtk_widget_show(launcher->hbox);
+	gtk_widget_show(launcher->box);
+	g_signal_connect(launcher->box, "button_press_event", G_CALLBACK(launcher_clicked), launcher);
+	g_signal_connect(launcher->box, "button-release-event", G_CALLBACK(launcher_clicked), launcher);
+	g_signal_connect(launcher->box, "enter-notify-event", G_CALLBACK(launcher_passthrough), launcher);
+	g_signal_connect(launcher->box, "leave-notify-event", G_CALLBACK(launcher_passthrough), launcher);
 }
 
-t_launcher *
-launcher_new (const gchar *command, gint icon_id, const gchar *icon_name, t_quicklauncher* quicklauncher)
+
+t_launcher *launcher_new (const gchar *name, const gchar *command, gint icon_id,
+						const gchar *icon_name, t_quicklauncher* quicklauncher)
 {
-    t_launcher *launcher;
+	t_launcher *launcher;
 	launcher = g_new0 (t_launcher, 1);
-	
-    if(command)
-	{
-		launcher->command = g_malloc( (strlen(command)+1)*sizeof(gchar) );
-		launcher->command = strcpy(launcher->command, command);
-	}
+	if(name)
+		launcher->name= g_strdup(name);
+	else 
+		launcher->name= NULL;
+	if(command)
+		launcher->command = g_strdup(command);
 	else launcher->command = NULL;
 	launcher->icon_id = icon_id;
 	if (icon_name)
-	{
-		launcher->icon_name = g_malloc( (strlen(icon_name)+1)*sizeof(gchar) );
-		launcher->icon_name =strcpy(launcher->icon_name, icon_name);
-	}
-	else launcher->icon_name = NULL; 	
+		launcher->icon_name = g_strdup(icon_name);
+	else launcher->icon_name = NULL;
 	launcher->quicklauncher = quicklauncher;
 	create_launcher(launcher);
 	return launcher;
@@ -539,15 +595,12 @@ launcher_free (t_launcher *launcher)
 	if(!launcher) return;
 	UNREF(launcher->def_img);
 	UNREF(launcher->zoomed_img);
-	UNREF(launcher->clicked_img); 
-	//g_object_unref(launcher->tooltip);
-	g_object_unref(launcher->widget);
+	UNREF(launcher->clicked_img);
+	UNREF(launcher->tooltip);
+	g_object_unref(launcher->box);
 	g_object_unref(launcher->image);
-	
-	//gtk_widget_destroy(launcher->widget); //useless: handled by gtk
 	g_free(launcher->icon_name);
 	g_free(launcher->command);
-	
     g_free (launcher);
 }
 
@@ -556,26 +609,28 @@ launcher_load_config(XfceRc *rcfile, gint num, t_quicklauncher *quicklauncher)
 {
 	char group[15];
 	t_launcher *launcher;
-	g_sprintf(group, "launcher_%d%c", num,0); 
+	g_sprintf(group, "launcher_%d%c", num,0);
 	xfce_rc_set_group(rcfile, group);
-	
-	launcher = g_new0 (t_launcher, 1);
-	launcher->quicklauncher = quicklauncher;
-	launcher->command = g_strdup(xfce_rc_read_entry(rcfile, "command", NULL));
-	launcher->icon_name = g_strdup(xfce_rc_read_entry(rcfile, "icon_name", NULL));
-	launcher->icon_id = xfce_rc_read_int_entry(rcfile, "icon_id", 0);
-	
-	create_launcher(launcher);
+	launcher = launcher_new(xfce_rc_read_entry(rcfile, "name", NULL),
+							xfce_rc_read_entry(rcfile, "command", NULL),
+							xfce_rc_read_int_entry(rcfile, "icon_id", 0),
+							xfce_rc_read_entry(rcfile, "icon_name", NULL),
+							quicklauncher);
 	return launcher;
 }
 
-void 
+void
 launcher_save_config(t_launcher *launcher, XfceRc *rcfile, guint16 num)
 {
 	char group[15];
-	g_sprintf(group, "launcher_%d%c", num,0); 
+	DBG("\t launcher %d: %s", num, launcher->command);
+	g_sprintf(group, "launcher_%d%c", num,0);
+	xfce_rc_delete_group(rcfile, group, FALSE);
 	xfce_rc_set_group(rcfile, group);
-	xfce_rc_write_entry(rcfile, "command", launcher->command);
+	if(launcher->command)
+		xfce_rc_write_entry(rcfile, "command", launcher->command);
+	if(launcher->name)
+		xfce_rc_write_entry(rcfile, "name", launcher->name);
 	if(launcher->icon_name)
 		xfce_rc_write_entry(rcfile, "icon_name", launcher->icon_name);
 	xfce_rc_write_int_entry(rcfile, "icon_id", launcher->icon_id);
